package main

import (
	"fmt"
	"io"
	"strings"
)

// writeTree prints a map of parent:children as an ASCII tree hierarchy
// 53 LOC, just for pretty-print, which seems like it should be shorter.
func writeTree(out io.Writer, tree map[string][]string) {
	// │ ├ ┌ └ ─ ┬ ┼
	var writeMe func([]string, string, bool)
	writeMe = func(cols []string, name string, isLast bool) {
		myChildren := tree[name]
		colsC := make([]string, len(cols)+2)
		for i, c := range cols {
			switch c {
			case "":
				colsC[i] = "┌"
			case "│", "├":
				colsC[i] = "│"
			case "┌":
				if isLast {
					colsC[i] = "└"
				} else {
					colsC[i] = "├"
				}
			case "─":
				prev := colsC[i-1]
				if prev == "├" || prev == "└" || prev == "─" || prev == "┌" {
					colsC[i] = "─"
				} else {
					colsC[i] = " "
				}
			case "┬":
				if isLast {
					colsC[i] = "└"
				} else {
					colsC[i] = "├"
				}
			default:
				colsC[i] = " "
			}
		}
		if len(myChildren) > 0 {
			if len(cols) == 0 {
				colsC[len(colsC)-2] = "┌"
			} else {
				colsC[len(colsC)-2] = "┬"
			}
		}
		colsC[len(colsC)-1] = "─"
		fmt.Fprintf(out, "%s %s\n", strings.Join(colsC, ""), name)
		for i, child := range myChildren {
			if i < len(myChildren)-1 {
				writeMe(colsC, child, false)
			} else {
				writeMe(colsC, child, true)
			}
		}
	}
	writeMe([]string{}, "/", false)
}
